@@ -2,15 +2,30 @@
 # 2.0, and the BSD License. See the LICENSE file in the root of this repository
 # for complete details.
 
-from __future__ import absolute_import, division, print_function
 
 import abc
-
-import six
+import typing
 
 
-@six.add_metaclass(abc.ABCMeta)
-class CipherBackend(object):
+if typing.TYPE_CHECKING:
+    from cryptography.hazmat.primitives.asymmetric.types import (
+        PRIVATE_KEY_TYPES,
+    )
+    from cryptography.hazmat.primitives import hashes
+    from cryptography.x509.base import (
+        Certificate,
+        CertificateBuilder,
+        CertificateRevocationList,
+        CertificateRevocationListBuilder,
+        CertificateSigningRequest,
+        CertificateSigningRequestBuilder,
+        RevokedCertificate,
+        RevokedCertificateBuilder,
+    )
+    from cryptography.x509.name import Name
+
+
+class CipherBackend(metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def cipher_supported(self, cipher, mode):
         """
@@ -30,8 +45,7 @@ class CipherBackend(object):
         """
 
 
-@six.add_metaclass(abc.ABCMeta)
-class HashBackend(object):
+class HashBackend(metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def hash_supported(self, algorithm):
         """
@@ -45,8 +59,7 @@ class HashBackend(object):
         """
 
 
-@six.add_metaclass(abc.ABCMeta)
-class HMACBackend(object):
+class HMACBackend(metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def hmac_supported(self, algorithm):
         """
@@ -57,12 +70,11 @@ class HMACBackend(object):
     @abc.abstractmethod
     def create_hmac_ctx(self, key, algorithm):
         """
-        Create a MACContext for calculating a message authentication code.
+        Create a context for calculating a message authentication code.
         """
 
 
-@six.add_metaclass(abc.ABCMeta)
-class CMACBackend(object):
+class CMACBackend(metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def cmac_algorithm_supported(self, algorithm):
         """
@@ -72,12 +84,11 @@ class CMACBackend(object):
     @abc.abstractmethod
     def create_cmac_ctx(self, algorithm):
         """
-        Create a MACContext for calculating a message authentication code.
+        Create a context for calculating a message authentication code.
         """
 
 
-@six.add_metaclass(abc.ABCMeta)
-class PBKDF2HMACBackend(object):
+class PBKDF2HMACBackend(metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def pbkdf2_hmac_supported(self, algorithm):
         """
@@ -86,15 +97,15 @@ class PBKDF2HMACBackend(object):
         """
 
     @abc.abstractmethod
-    def derive_pbkdf2_hmac(self, algorithm, length, salt, iterations,
-                           key_material):
+    def derive_pbkdf2_hmac(
+        self, algorithm, length, salt, iterations, key_material
+    ):
         """
         Return length bytes derived from provided PBKDF2 parameters.
         """
 
 
-@six.add_metaclass(abc.ABCMeta)
-class RSABackend(object):
+class RSABackend(metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def generate_rsa_private_key(self, public_exponent, key_size):
         """
@@ -128,8 +139,7 @@ class RSABackend(object):
         """
 
 
-@six.add_metaclass(abc.ABCMeta)
-class DSABackend(object):
+class DSABackend(metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def generate_dsa_parameters(self, key_size):
         """
@@ -180,8 +190,7 @@ class DSABackend(object):
         """
 
 
-@six.add_metaclass(abc.ABCMeta)
-class EllipticCurveBackend(object):
+class EllipticCurveBackend(metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def elliptic_curve_signature_algorithm_supported(
         self, signature_algorithm, curve
@@ -212,12 +221,23 @@ class EllipticCurveBackend(object):
     @abc.abstractmethod
     def load_elliptic_curve_private_numbers(self, numbers):
         """
-        Return an EllipticCurvePublicKey provider using the given numbers.
+        Return an EllipticCurvePrivateKey provider using the given numbers.
+        """
+
+    @abc.abstractmethod
+    def elliptic_curve_exchange_algorithm_supported(self, algorithm, curve):
+        """
+        Returns whether the exchange algorithm is supported by this backend.
+        """
+
+    @abc.abstractmethod
+    def derive_elliptic_curve_private_key(self, private_value, curve):
+        """
+        Compute the private key given the private value and curve.
         """
 
 
-@six.add_metaclass(abc.ABCMeta)
-class PEMSerializationBackend(object):
+class PEMSerializationBackend(metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def load_pem_private_key(self, data, password):
         """
@@ -231,9 +251,14 @@ class PEMSerializationBackend(object):
         Loads a public key from PEM encoded data.
         """
 
+    @abc.abstractmethod
+    def load_pem_parameters(self, data):
+        """
+        Load encryption parameters from PEM encoded data.
+        """
 
-@six.add_metaclass(abc.ABCMeta)
-class DERSerializationBackend(object):
+
+class DERSerializationBackend(metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def load_der_private_key(self, data, password):
         """
@@ -247,40 +272,70 @@ class DERSerializationBackend(object):
         Loads a public key from DER encoded data.
         """
 
-
-@six.add_metaclass(abc.ABCMeta)
-class X509Backend(object):
     @abc.abstractmethod
-    def load_pem_x509_certificate(self, data):
+    def load_der_parameters(self, data):
         """
-        Load an X.509 certificate from PEM encoded data.
+        Load encryption parameters from DER encoded data.
+        """
+
+
+class X509Backend(metaclass=abc.ABCMeta):
+    @abc.abstractmethod
+    def create_x509_csr(
+        self,
+        builder: "CertificateSigningRequestBuilder",
+        private_key: "PRIVATE_KEY_TYPES",
+        algorithm: typing.Optional["hashes.HashAlgorithm"],
+    ) -> "CertificateSigningRequest":
+        """
+        Create and sign an X.509 CSR from a CSR builder object.
         """
 
     @abc.abstractmethod
-    def load_der_x509_certificate(self, data):
+    def create_x509_certificate(
+        self,
+        builder: "CertificateBuilder",
+        private_key: "PRIVATE_KEY_TYPES",
+        algorithm: typing.Optional["hashes.HashAlgorithm"],
+    ) -> "Certificate":
         """
-        Load an X.509 certificate from DER encoded data.
+        Create and sign an X.509 certificate from a CertificateBuilder object.
         """
 
     @abc.abstractmethod
-    def load_der_x509_csr(self, data):
+    def create_x509_crl(
+        self,
+        builder: "CertificateRevocationListBuilder",
+        private_key: "PRIVATE_KEY_TYPES",
+        algorithm: typing.Optional["hashes.HashAlgorithm"],
+    ) -> "CertificateRevocationList":
         """
-        Load an X.509 CSR from DER encoded data.
+        Create and sign an X.509 CertificateRevocationList from a
+        CertificateRevocationListBuilder object.
         """
 
     @abc.abstractmethod
-    def load_pem_x509_csr(self, data):
+    def create_x509_revoked_certificate(
+        self, builder: "RevokedCertificateBuilder"
+    ) -> "RevokedCertificate":
         """
-        Load an X.509 CSR from PEM encoded data.
+        Create a RevokedCertificate object from a RevokedCertificateBuilder
+        object.
         """
 
-
-@six.add_metaclass(abc.ABCMeta)
-class DHBackend(object):
     @abc.abstractmethod
-    def generate_dh_parameters(self, key_size):
+    def x509_name_bytes(self, name: "Name") -> bytes:
+        """
+        Compute the DER encoded bytes of an X509 Name object.
+        """
+
+
+class DHBackend(metaclass=abc.ABCMeta):
+    @abc.abstractmethod
+    def generate_dh_parameters(self, generator, key_size):
         """
         Generate a DHParameters instance with a modulus of key_size bits.
+        Using the given generator. Often 2 or 5.
         """
 
     @abc.abstractmethod
@@ -291,37 +346,104 @@ class DHBackend(object):
         """
 
     @abc.abstractmethod
-    def generate_dh_private_key_and_parameters(self, key_size):
+    def generate_dh_private_key_and_parameters(self, generator, key_size):
         """
         Generate a DHPrivateKey instance using key size only.
+        Using the given generator. Often 2 or 5.
         """
 
     @abc.abstractmethod
     def load_dh_private_numbers(self, numbers):
         """
-        Returns a DHPrivateKey provider.
+        Load a DHPrivateKey from DHPrivateNumbers
         """
 
     @abc.abstractmethod
     def load_dh_public_numbers(self, numbers):
         """
-        Returns a DHPublicKey provider.
+        Load a DHPublicKey from DHPublicNumbers.
         """
 
     @abc.abstractmethod
     def load_dh_parameter_numbers(self, numbers):
         """
-        Returns a DHParameters provider.
+        Load DHParameters from DHParameterNumbers.
         """
 
     @abc.abstractmethod
-    def dh_exchange_algorithm_supported(self, exchange_algorithm):
-        """
-        Returns whether the exchange algorithm is supported by this backend.
-        """
-
-    @abc.abstractmethod
-    def dh_parameters_supported(self, p, g):
+    def dh_parameters_supported(self, p, g, q=None):
         """
         Returns whether the backend supports DH with these parameter values.
         """
+
+    @abc.abstractmethod
+    def dh_x942_serialization_supported(self):
+        """
+        Returns True if the backend supports the serialization of DH objects
+        with subgroup order (q).
+        """
+
+
+class ScryptBackend(metaclass=abc.ABCMeta):
+    @abc.abstractmethod
+    def derive_scrypt(self, key_material, salt, length, n, r, p):
+        """
+        Return bytes derived from provided Scrypt parameters.
+        """
+
+    @abc.abstractmethod
+    def scrypt_supported(self):
+        """
+        Return True if Scrypt is supported.
+        """
+
+
+# This is the catch-all for future backend methods and inherits all the
+# other interfaces as well so we can just use Backend for typing.
+class Backend(
+    CipherBackend,
+    CMACBackend,
+    DERSerializationBackend,
+    DHBackend,
+    DSABackend,
+    EllipticCurveBackend,
+    HashBackend,
+    HMACBackend,
+    PBKDF2HMACBackend,
+    RSABackend,
+    PEMSerializationBackend,
+    ScryptBackend,
+    X509Backend,
+    metaclass=abc.ABCMeta,
+):
+    @abc.abstractmethod
+    def load_pem_pkcs7_certificates(self, data):
+        """
+        Returns a list of x509.Certificate
+        """
+
+    @abc.abstractmethod
+    def load_der_pkcs7_certificates(self, data):
+        """
+        Returns a list of x509.Certificate
+        """
+
+    @abc.abstractmethod
+    def pkcs7_sign(self, builder, encoding, options):
+        """
+        Returns bytes
+        """
+
+    @abc.abstractmethod
+    def load_key_and_certificates_from_pkcs12(self, data, password):
+        """
+        Returns a tuple of (key, cert, [certs])
+        """
+
+    @abc.abstractmethod
+    def serialize_key_and_certificates_to_pkcs12(
+        self, name, key, cert, cas, encryption_algorithm
+    ):
+        """
+        Returns bytes
+        """
